@@ -8,6 +8,7 @@ require 'rake/rdoctask'
 require 'rake/testtask'
 require 'rbconfig'
 require 'rubyforge'
+require 'yaml'
 
 ##
 # hoe - a tool to help rake
@@ -33,31 +34,89 @@ require 'rubyforge'
 #
 # === Tasks Provided:
 #
-# * announce         - Generate email announcement file and post to rubyforge.
-# * audit            - Run ZenTest against the package
-# * check_manifest   - Verify the manifest
-# * clean            - Clean up all the extras
-# * config_hoe       - Create a fresh ~/.hoerc file
-# * debug_gem        - Show information about the gem.
-# * default          - Run the default tasks
-# * docs             - Build the docs HTML Files
-# * email            - Generate email announcement file.
-# * gem              - Build the gem file only.
-# * install          - Install the package. Uses PREFIX and RUBYLIB
-# * install_gem      - Install the package as a gem
-# * multi            - Run the test suite using multiruby
-# * package          - Build all the packages
-# * post_blog        - Post announcement to blog.
-# * post_news        - Post announcement to rubyforge.
-# * publish_docs     - Publish RDoc to RubyForge
-# * release          - Package and upload the release to rubyforge.
-# * ridocs           - Generate ri locally for testing
-# * test             - Run the test suite. Use FILTER to add to the command line.
-# * test_deps        - Show which test files fail when run alone.
-# * uninstall        - Uninstall the package.
+# announce::       Create news email file and post to rubyforge.
+# audit::          Run ZenTest against the package.
+# check_manifest:: Verify the manifest.
+# clean::          Clean up all the extras.
+# config_hoe::     Create a fresh ~/.hoerc file.
+# debug_gem::      Show information about the gem.
+# default::        Run the default tasks.
+# docs::           Build the docs HTML Files
+# email::          Generate email announcement file.
+# gem::            Build the gem file hoe-1.5.0.gem
+# generate_key::   Generate a key for signing your gems.
+# install_gem::    Install the package as a gem.
+# multi::          Run the test suite using multiruby.
+# package::        Build all the packages
+# post_blog::      Post announcement to blog.
+# post_news::      Post announcement to rubyforge.
+# publish_docs::   Publish RDoc to RubyForge.
+# release::        Package and upload the release to rubyforge.
+# ridocs::         Generate ri locally for testing.
+# test::           Run the test suite.
+# test_deps::      Show which test files fail when run alone.
 #
+# === Extra Configuration Options:
+#
+# Run +config_hoe+ to generate a new ~/.hoerc file. The file is a
+# YAML formatted config file with the following settings:
+#
+# exclude::             A regular expression of files to exclude from
+#                       +check_manifest+.
+# publish_on_announce:: Run +publish_docs+ when you run +release+.
+# signing_key_file:: Signs your gems with this private key.
+# signing_cert_file:: Signs your gem with this certificate.
+# blogs::               An array of hashes of blog settings.
+#
+# Run +config_hoe+ and see ~/.hoerc for examples.
+#
+# === Signing Gems:
+#
+# Run the 'generate_key' task.  This will:
+#
+# 1. Configure your ~/.hoerc.
+# 2. Generate a signing key and certificate.
+# 3. Install the private key and public certificate files into ~/.gem.
+# 4. Upload the certificate to RubyForge.
+#
+# Hoe will now generate signed gems when the package task is run.  If you have
+# multiple machines you build gems on, be sure to install your key and
+# certificate on each machine.
+#
+# Keep your private key secret!  Keep your private key safe!
+#
+# To make sure your gems are signed run:
+#
+#   rake package; tar tf pkg/yourproject-1.2.3.gem
+#
+# If your gem is signed you will see:
+#
+#   data.tar.gz
+#   data.tar.gz.sig
+#   metadata.gz
+#   metadata.gz.sig
+#
+# === Platform awareness
+#
+# Hoe allows bundling of pre-compiled extensions in the +package+ task.
+#
+# To create a package for your current platform:
+#
+#   rake package INLINE=1
+#
+# This will force Hoe analize your +Inline+ already compiled
+# extensions and include them in your gem.
+#
+# If somehow you need to force a specific platform:
+#
+#   rake package INLINE=1 FORCE_PLATFORM=mswin32
+#
+# This will set the +Gem::Specification+ platform to the one indicated in
+# +FORCE_PLATFORM+ (instead of default Gem::Platform::CURRENT)
+#
+
 class Hoe
-  VERSION = '1.2.1'
+  VERSION = '1.5.1'
 
   ruby_prefix = Config::CONFIG['prefix']
   sitelibdir = Config::CONFIG['sitelibdir']
@@ -93,7 +152,9 @@ class Hoe
               File.join(PREFIX, sitelibdir[ruby_prefix.size..-1])
             end
 
-  WINDOZE = /win32/ =~ RUBY_PLATFORM unless defined? WINDOZE
+  DLEXT = Config::CONFIG['DLEXT']
+
+  WINDOZE = /djgpp|(cyg|ms|bcc)win|mingw/ =~ RUBY_PLATFORM unless defined? WINDOZE
 
   DIFF = if WINDOZE
            'diff.exe'
@@ -119,7 +180,7 @@ class Hoe
   attr_accessor :bin_files # :nodoc:
 
   ##
-  # *Recommended*: A description of the release's latest changes.
+  # Optional: A description of the release's latest changes. Auto-populates.
 
   attr_accessor :changes
 
@@ -129,10 +190,15 @@ class Hoe
   attr_accessor :clean_globs
 
   ##
-  # *Recommended*: A description of the project.
+  # Optional: A description of the project. Auto-populates.
 
   attr_accessor :description
 
+  ##
+  # Optional: What sections from the readme to use for auto-description. Defaults to %w(description).
+
+  attr_accessor :description_sections
+
   ##
   # *Recommended*: The author's email address(es). (can be array)
 
@@ -194,10 +260,15 @@ class Hoe
   attr_accessor :spec_extras
 
   ##
-  # *Recommended*: A short summary of the project.
+  # Optional: A short summary of the project. Auto-populates.
 
   attr_accessor :summary
 
+  ##
+  # Optional: Number of sentences from description for summary. Defaults to 1.
+
+  attr_accessor :summary_sentences
+
   ##
   # Populated automatically from the manifest. List of tests.
 
@@ -209,7 +280,7 @@ class Hoe
   attr_accessor :test_globs
 
   ##
-  # *Recommended*: The url(s) of the project. (can be array)
+  # Optional: The url(s) of the project. (can be array). Auto-populates.
 
   attr_accessor :url
 
@@ -223,36 +294,66 @@ class Hoe
     self.version = version
 
     # Defaults
-    self.author = "Ryan Davis"
-    self.changes = "The author was too lazy to write a changeset"
-    self.clean_globs = %w(diff diff.txt email.txt ri *.gem **/*~)
-    self.description = "The author was too lazy to write a description"
-    self.email = "ryand-ruby@zenspider.com"
+    self.author = []
+    self.clean_globs = %w(diff diff.txt email.txt ri
+                          *.gem *~ **/*~ *.rbc **/*.rbc)
+    self.description_sections = %w(description)
+    self.email = []
     self.extra_deps = []
     self.need_tar = true
     self.need_zip = false
-    self.rdoc_pattern = /^(lib|bin)|txt$/
+    self.rdoc_pattern = /^(lib|bin|ext)|txt$/
     self.remote_rdoc_dir = name
     self.rsync_args = '-av --delete'
     self.rubyforge_name = name.downcase
     self.spec_extras = {}
-    self.summary = "The author was too lazy to write a summary"
+    self.summary_sentences = 1
     self.test_globs = ['test/**/test_*.rb']
-    self.url = "http://www.zenspider.com/ZSS/Products/#{name}/"
 
     yield self if block_given?
 
+    # Intuit values:
+
+    readme   = File.read("README.txt").split(/^(=+ .*)$/)[1..-1]
+    unless readme.empty? then
+      sections = readme.map { |s|
+        s =~ /^=/ ? s.strip.downcase.chomp(':').split.last : s.strip
+      }
+      sections = Hash[*sections]
+      desc = sections.values_at(*description_sections).join("\n\n")
+      summ = desc.split(/\.\s+/).first(summary_sentences).join(". ")
+
+      self.description ||= desc
+      self.changes ||= File.read("History.txt").split(/^(===.*)/)[1..2].join.strip
+      self.summary ||= summ
+      self.url ||= readme[1].gsub(/^\* /, '').split(/\n/).grep(/\S+/)
+    else
+      warn "** README.txt is in the wrong format for auto-intuiting."
+      warn "   run sow blah and look at it's text files"
+    end
+
+    %w(email author).each do |field|
+      value = self.send(field)
+      if value.nil? or value.empty? then
+        if Time.now < Time.local(2008, 4, 1) then
+          warn "Hoe #{field} value not set - Fix by 2008-04-01!"
+          self.send "#{field}=", "doofus"
+        else
+          abort "Hoe #{field} value not set"
+        end
+      end
+    end
+
     hoe_deps = {
       'rake' => ">= #{RAKEVERSION}",
       'rubyforge' => ">= #{::RubyForge::VERSION}",
     }
 
-    self.extra_deps = Array(extra_deps) # just in case user used = instead of <<
-    self.extra_deps = [extra_deps] unless
-      extra_deps.empty? or Array === extra_deps.first
+    self.extra_deps = Array(extra_deps).map { |o| String === o ? [o] : o }
+
     if name == 'hoe' then
-      hoe_deps.each do |pkg, version|
-        extra_deps << [pkg, version]
+      hoe_deps.each do |pkg, vers|
+        extra_deps << [pkg, vers]
       end
     else
       extra_deps << ['hoe', ">= #{VERSION}"] unless hoe_deps.has_key? name
@@ -261,8 +362,20 @@ class Hoe
     define_tasks
   end
 
+  def developer name, email
+    self.author << name
+    self.email << email
+  end
+
   def define_tasks # :nodoc:
-    desc 'Run the default tasks'
+    def with_config # :nodoc:
+      rc = File.expand_path("~/.hoerc")
+      exists = File.exist? rc
+      config = exists ? YAML.load_file(rc) : {}
+      yield(config, rc)
+    end
+
+    desc 'Run the default tasks.'
     task :default => :test
 
     desc 'Run the test suite. Use FILTER to add to the command line.'
@@ -281,7 +394,7 @@ class Hoe
       end
     end
 
-    desc 'Run the test suite using multiruby'
+    desc 'Run the test suite using multiruby.'
     task :multi do
       run_tests :multi
     end
@@ -289,6 +402,18 @@ class Hoe
     ############################################################
     # Packaging and Installing
 
+    signing_key = nil
+    cert_chain = []
+
+    with_config do |config, path|
+      break unless config['signing_key_file'] and config['signing_cert_file']
+      key_file = File.expand_path config['signing_key_file'].to_s
+      signing_key = key_file if File.exist? key_file
+
+      cert_file = File.expand_path config['signing_cert_file'].to_s
+      cert_chain << cert_file if File.exist? cert_file
+    end
+
     self.spec = Gem::Specification.new do |s|
       s.name = name
       s.version = version
@@ -326,6 +451,43 @@ class Hoe
         s.test_files = Dir[*test_globs]
       end
 
+      if signing_key and cert_chain then
+        s.signing_key = signing_key
+        s.cert_chain = cert_chain
+      end
+
+      ############################################################
+      # Allow automatic inclusion of compiled extensions
+      if ENV['INLINE'] then
+        s.platform = ENV['FORCE_PLATFORM'] || Gem::Platform::CURRENT
+        # name of the extension is CamelCase
+        if name =~ /[A-Z]/
+          # ClassName => class_name
+          alternate_name = name.reverse.scan(%r/[A-Z]+|[^A-Z]*[A-Z]+?/).reverse.map { |word| word.reverse.downcase }.join('_')
+        elsif name =~ /_/
+          # class_name = ClassName
+          alternate_name = name.strip.split(/\s*_+\s*/).map! { |w| w.downcase.sub(/^./) { |c| c.upcase } }.join
+        end
+
+        # Try collecting Inline extensions for +name+
+        if defined?(Inline) then
+          directory 'lib/inline'
+
+          extensions = Dir.chdir(Inline::directory) {
+            Dir["Inline_{#{name},#{alternate_name}}_*.#{DLEXT}"]
+          }
+          extensions.each do |ext|
+            # add the inlined extension to the spec files
+            s.files += ["lib/inline/#{ext}"]
+
+            # include the file in the tasks
+            file "lib/inline/#{ext}" => ["lib/inline"] do
+              cp File.join(Inline::directory, ext), "lib/inline"
+            end
+          end
+        end
+      end
+
       # Do any extra stuff the user wants
       spec_extras.each do |msg, val|
         case val
@@ -351,32 +513,9 @@ class Hoe
       pkg.need_zip = @need_zip
     end
 
-    desc 'Install the package. Uses PREFIX and RUBYLIB'
-    task :install do
-      [
-       [lib_files + test_files, RUBYLIB, 0444],
-       [bin_files, File.join(PREFIX, 'bin'), 0555]
-      ].each do |files, dest, mode|
-        FileUtils.mkdir_p dest unless test ?d, dest
-        files.each do |file|
-          install file, dest, :mode => mode
-        end
-      end
-    end
-
-    desc 'Install the package as a gem'
+    desc 'Install the package as a gem.'
     task :install_gem => [:clean, :package] do
-      sh "sudo gem install pkg/*.gem"
-    end
-
-    desc 'Uninstall the package.'
-    task :uninstall do
-      Dir.chdir RUBYLIB do
-        rm_f((lib_files + test_files).map { |f| File.basename f })
-      end
-      Dir.chdir File.join(PREFIX, 'bin') do
-        rm_f bin_files.map { |f| File.basename f }
-      end
+      sh "#{'sudo ' unless WINDOZE}gem install --local pkg/*.gem"
     end
 
     desc 'Package and upload the release to rubyforge.'
@@ -412,24 +551,24 @@ class Hoe
 
     Rake::RDocTask.new(:docs) do |rd|
       rd.main = "README.txt"
-      rd.options << '-d' if RUBY_PLATFORM !~ /win32/ and `which dot` =~ /\/dot/
+      rd.options << '-d' if RUBY_PLATFORM !~ /win32/ and `which dot` =~ /\/dot/ and not ENV['NODOT']
       rd.rdoc_dir = 'doc'
       files = spec.files.grep(rdoc_pattern)
       files -= ['Manifest.txt']
       rd.rdoc_files.push(*files)
 
       title = "#{name}-#{version} Documentation"
-      title = "#{rubyforge_name}'s " + title if rubyforge_name != title
+      title = "#{rubyforge_name}'s " + title if rubyforge_name != name
 
       rd.options << "-t #{title}"
     end
 
-    desc "Generate ri locally for testing"
+    desc 'Generate ri locally for testing.'
     task :ridocs => :clean do
       sh %q{ rdoc --ri -o ri . }
     end
 
-    desc "Publish RDoc to RubyForge"
+    desc 'Publish RDoc to RubyForge.'
     task :publish_docs => [:clean, :docs] do
       config = YAML.load(File.read(File.expand_path("~/.rubyforge/user-config.yml")))
       host = "#{config["username"]}@rubyforge.org"
@@ -442,39 +581,21 @@ class Hoe
 
     # no doco for this one
     task :publish_on_announce do
-      with_config do |rc, path|
-        if rc["publish_on_announce"] then
-          Rake::Task['publish_docs'].invoke
-        end
+      with_config do |config, _|
+        Rake::Task['publish_docs'].invoke if config["publish_on_announce"]
       end
     end
 
     ############################################################
     # Misc/Maintenance:
 
-    def with_config(create=false) # :nodoc:
-      require 'yaml'
-      rc = File.expand_path("~/.hoerc")
-
-      unless create then
-        if test ?f, rc then
-          config = YAML.load_file(rc)
-          yield(config, rc)
-        end
-      else
-        unless test ?f, rc then
-          yield(rc)
-        end
-      end
-    end
-
-    desc 'Run ZenTest against the package'
+    desc 'Run ZenTest against the package.'
     task :audit do
       libs = %w(lib test ext).join(File::PATH_SEPARATOR)
       sh "zentest -I=#{libs} #{spec.files.grep(/^(lib|test)/).join(' ')}"
     end
 
-    desc 'Clean up all the extras'
+    desc 'Clean up all the extras.'
     task :clean => [ :clobber_docs, :clobber_package ] do
       clean_globs.each do |pattern|
         files = Dir[pattern]
@@ -482,11 +603,14 @@ class Hoe
       end
     end
 
-    desc 'Create a fresh ~/.hoerc file'
+    desc 'Create a fresh ~/.hoerc file.'
     task :config_hoe do
-      with_config(:create) do |rc, path|
-        blog = {
+      with_config do |config, path|
+        default_config = {
+          "exclude" => /tmp$|CVS|\.svn/,
           "publish_on_announce" => false,
+          "signing_key_file" => "~/.gem/gem-private_key.pem",
+          "signing_cert_file" => "~/.gem/gem-public_cert.pem",
           "blogs" => [ {
                          "user" => "user",
                          "url" => "url",
@@ -497,14 +621,12 @@ class Hoe
                          "password"=>"password",
                        } ],
         }
-        File.open(rc, "w") do |f|
-          YAML.dump(blog, f)
+        File.open(path, "w") do |f|
+          YAML.dump(default_config.merge(config), f)
         end
-      end
 
-      with_config do |rc, path|
         editor = ENV['EDITOR'] || 'vi'
-        system "#{editor} #{path}"
+        system "#{editor} #{path}" if ENV['SHOW_EDITOR'] != 'no'
       end
     end
 
@@ -532,8 +654,9 @@ class Hoe
       require 'xmlrpc/client'
 
       with_config do |config, path|
-        subject, title, body, urls = announcement
+        break unless config['blogs']
 
+        subject, title, body, urls = announcement
         body += "\n\n#{urls}"
 
         config['blogs'].each do |site|
@@ -561,23 +684,76 @@ class Hoe
       puts "Posted to rubyforge"
     end
 
-    desc 'Generate email announcement file and post to rubyforge.'
+    desc 'Create news email file and post to rubyforge.'
     task :announce => [:email, :post_news, :post_blog, :publish_on_announce ]
 
-    desc "Verify the manifest"
+    desc 'Verify the manifest.'
     task :check_manifest => :clean do
       f = "Manifest.tmp"
       require 'find'
       files = []
-      Find.find '.' do |path|
-        next unless File.file? path
-        next if path =~ /\.svn|tmp$|CVS/
-        files << path[2..-1]
+      with_config do |config, _|
+        exclusions = config["exclude"] || /tmp$|CVS|\.svn/
+        Find.find '.' do |path|
+          next unless File.file? path
+          next if path =~ exclusions
+          files << path[2..-1]
+        end
+        files = files.sort.join "\n"
+        File.open f, 'w' do |fp| fp.puts files end
+        system "#{DIFF} -du Manifest.txt #{f}"
+        rm f
+      end
+    end
+
+    desc 'Generate a key for signing your gems.'
+    task :generate_key do
+      email = spec.email
+      abort "No email in your gemspec" if email.nil? or email.empty?
+
+      key_file = with_config { |config, _| config['signing_key_file'] }
+      cert_file = with_config { |config, _| config['signing_cert_file'] }
+
+      if key_file.nil? or cert_file.nil? then
+        ENV['SHOW_EDITOR'] ||= 'no'
+        Rake::Task['config_hoe'].invoke
+
+        key_file = with_config { |config, _| config['signing_key_file'] }
+        cert_file = with_config { |config, _| config['signing_cert_file'] }
+      end
+
+      key_file = File.expand_path key_file
+      cert_file = File.expand_path cert_file
+
+      unless File.exist? key_file or File.exist? cert_file then
+        sh "gem cert --build #{email}"
+        mv "gem-private_key.pem", key_file, :verbose => true
+        mv "gem-public_cert.pem", cert_file, :verbose => true
+
+        puts "Installed key and certificate."
+
+        rf = RubyForge.new
+        rf.login
+
+        cert_package = "#{rubyforge_name}-certificates"
+
+        begin
+          rf.lookup 'package', cert_package
+        rescue
+          rf.create_package rubyforge_name, cert_package
+        end
+
+        begin
+          rf.lookup('release', cert_package)['certificates']
+          rf.add_file rubyforge_name, cert_package, 'certificates', cert_file
+        rescue
+          rf.add_release rubyforge_name, cert_package, 'certificates', cert_file
+        end
+
+        puts "Uploaded certificate to release \"certificates\" in package #{cert_package}"
+      else
+        puts "Keys already exist."
       end
-      files = files.sort.join "\n"
-      File.open f, 'w' do |fp| fp.puts files end
-      system "#{DIFF} -du Manifest.txt #{f}"
-      rm f
     end
 
   end # end define
@@ -598,7 +774,7 @@ class Hoe
     cmd = if test ?f, 'test/test_all.rb' then
             "#{RUBY_FLAGS} test/test_all.rb #{FILTER}"
           else
-            tests = test_globs.map { |g| Dir.glob(g) }.flatten << 'test/unit'
+            tests = ['test/unit'] + test_globs.map { |g| Dir.glob(g) }.flatten
             tests.map! {|f| %Q(require "#{f}")}
             "#{RUBY_FLAGS} -e '#{tests.join("; ")}' #{FILTER}"
           end
