@@ -14,7 +14,7 @@ class TestHoe < Test::Unit::TestCase
   # everything is forked out.
 
   def test_basics
-    boring   = %w(clobber_docs clobber_package redocs repackage)
+    boring   = %w(clobber_docs clobber_package gem redocs repackage)
     expected = %w(audit
                   announce
                   check_manifest
@@ -24,7 +24,7 @@ class TestHoe < Test::Unit::TestCase
                   default
                   docs
                   email
-                  install
+                  generate_key
                   install_gem
                   multi
                   package
@@ -34,11 +34,16 @@ class TestHoe < Test::Unit::TestCase
                   release
                   ridocs
                   test
-                  test_deps
-                  uninstall)
+                  test_deps)
     expected += boring
 
-    Hoe.new('blah', '1.0.0')
+    spec = Hoe.new('blah', '1.0.0') do |h|
+      h.developer("name", "email")
+    end
+
+    assert_equal ["name"], spec.author
+    assert_equal ["email"], spec.email
+
     tasks = Rake.application.tasks
     public_tasks = tasks.reject { |t| t.comment.nil? }.map { |t| t.name }.sort
 
